
public class ChainBidDemo {

  static Link setUpChain() {
    int NUM_LINKS = 8;
    Link first = new Link( 1 );
    for (int i=2; i <= NUM_LINKS; i++) first.addLast( new Link( i ) );
    return first;
  }

  public static void main( String[] args ) {
    int NUM_GOES= 10;
    Link chain = setUpChain();
    for (int i=0; i < NUM_GOES; i++) {
      System.out.print("\nContract "+i+ ":  ");
      chain.bid();                          // 3. Client "launches & leaves"
    }
  }  

}


class Link {
  private int  id;
  private Link next;                       // 1. "next" pointer
		          
  private static int  theBid = 99999;        // 4. Current bid and bidder
  private static Link bidder;

  public Link( int num ) {
    id = num;
  }

  public void addLast( Link p ) {
    if (next != null) next.addLast( p );  // 2. Handle and/or pass on
    else              next = p; 
  }

  public void bid() {
    int num = 1+ ((int)(Math.random() * 100000)) % 500 ; // my 'highest' bid
    System.out.print( id + "-" + num + "  " );
    if (num < theBid) {
      theBid = num;                      // 4. Current bid and bidder
      asp_Bidding.lowBid = num;
      bidder = this;
    }
    if (next != null) next.bid();         // 2. Handle and/or pass on
    else              bidder.execute();   // 5. The last 1 assigns the job
  }

  public void execute() {
    System.out.println( "\n >>> " + id + " wins the bid" );
    System.out.println( " >>> " + id + " is now doing the work\n" );
    System.out.println( "low bid: " + asp_Bidding.getLowBid());
    theBid = 999;
  }  
}

aspect asp_Bidding {
  static int lowBid = 99999;
	    
  public static int getLowBid() {
    return lowBid;
  }
	      
  pointcut Bidding():  call( void bid() );
	        
  before(): Bidding() {
    System.out.print("\nabout to bid: "+thisJoinPoint+"  ");
  }
		 
  after(): Bidding()  {
    System.out.println("did this bid: "+thisJoinPoint);
  }

//pointcut defined at advice point
  after(): call (Link setUpChain() ) {
    System.out.println("Set up bid chain: "+ thisJoinPoint);
  }

}

